@@ -1,4 +1,4 @@
-/* $Id: cut.c 5041 2014-06-30 20:39:27Z bens $ */
+/* $Id: cut.c 5393 2015-11-06 20:55:46Z bens $ */
 /**************************************************************************
  *   cut.c                                                                *
  *                                                                        *
@@ -50,11 +50,6 @@ inline bool keeping_cutbuffer(void)
  * current line. */
 void cut_line(void)
 {
-#ifndef NANO_TINY
-    if (!openfile->mark_begin)
-	openfile->mark_begin = openfile->current;
-#endif
-
     if (openfile->current != openfile->filebot)
 	move_to_filestruct(&cutbuffer, &cutbottom, openfile->current, 0,
 		openfile->current->next, 0);
@@ -124,7 +119,7 @@ void cut_to_eof(void)
  * position to the end of the file into the cutbuffer. */
 void do_cut_text(
 #ifndef NANO_TINY
-	bool copy_text, bool cut_till_eof, bool undoing
+	bool copy_text, bool cut_till_eof
 #else
 	void
 #endif
@@ -214,15 +209,13 @@ void do_cut_text(
 	 * disturbing the text. */
 	if (!old_no_newlines)
 	    UNSET(NO_NEWLINES);
-    } else if (!undoing)
-	update_undo(cut_till_eof ? CUT_EOF : CUT);
+    }
 
     /* Leave the text in the cutbuffer, and mark the file as
      * modified. */
-    if (!copy_text) {
-	set_modified();
-    }
+    if (!copy_text)
 #endif /* !NANO_TINY */
+	set_modified();
 
     /* Update the screen. */
     edit_refresh_needed = TRUE;
@@ -241,12 +234,11 @@ void do_cut_text_void(void)
 {
 #ifndef NANO_TINY
     add_undo(CUT);
+    do_cut_text(FALSE, FALSE);
+    update_undo(CUT);
+#else
+    do_cut_text();
 #endif
-    do_cut_text(
-#ifndef NANO_TINY
-	FALSE, FALSE, FALSE
-#endif
-	);
 }
 
 #ifndef NANO_TINY
@@ -261,7 +253,7 @@ void do_copy_text(void)
     if (mark_set || openfile->current != next_contiguous_line)
 	cutbuffer_reset();
 
-    do_cut_text(TRUE, FALSE, FALSE);
+    do_cut_text(TRUE, FALSE);
 
     /* If the mark was set, blow away the cutbuffer on the next copy. */
     next_contiguous_line = (mark_set ? NULL : openfile->current);
@@ -271,7 +263,8 @@ void do_copy_text(void)
 void do_cut_till_eof(void)
 {
     add_undo(CUT_EOF);
-    do_cut_text(FALSE, TRUE, FALSE);
+    do_cut_text(FALSE, TRUE);
+    update_undo(CUT_EOF);
 }
 #endif /* !NANO_TINY */
 
